@@ -24,6 +24,10 @@
 
 #include <X11/Xlib.h>
 
+#ifdef USE_XKB
+#include <X11/XKBlib.h>
+#endif
+
 #include <gtk/gtk.h>
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
@@ -33,6 +37,7 @@
 #include <libxfce4util/libxfce4util.h>
 #include <xfce-mcs-manager/manager-plugin.h>
 #include <libxfcegui4/libxfcegui4.h>
+#include <libxfcegui4/netk-trayicon.h>
 
 #include "mouse-plugin-internal.h"
 
@@ -53,6 +58,30 @@
 #define DEFAULT_ICON_SIZE 48
 #define DEFAULT_PTR_MAP_SIZE 128
 
+#ifdef USE_XKB
+static const char * mouse_tray_image_xpm[] = {
+"16 16 3 1",
+"       c None",
+".      c #000000",
+"+      c #FFFFFF",
+"                ",
+"     ......     ",
+"    .+.++.+.    ",
+"    .+.++.+.    ",
+"   .++.++.++.   ",
+"   .++.++.++.   ",
+"   ..........   ",
+"   .++++++++.   ",
+"   .++++++++.   ",
+"   .++++++++.   ",
+"   .++++++++.   ",
+"   .++++++++.   ",
+"   .++++++++.   ",
+"    .++++++.    ",
+"     ......     ",
+"                "};
+#endif
+
 static void create_channel(McsPlugin * mcs_plugin);
 static void run_dialog(McsPlugin * mcs_plugin);
 
@@ -64,6 +93,16 @@ static int acceleration = DEFAULT_ACCELERATION;
 static int threshold = DEFAULT_THRESHOLD;
 static int denominator = DEFAULT_DENOMINATOR;
 
+#ifdef USE_XKB
+static gboolean mouse_key = FALSE;
+static int mouse_keys_delay = 200;
+static int mouse_keys_interval = 200;
+static int mouse_keys_ttm = 200;
+static int mouse_keys_max_speed = 200;
+static gboolean xkbpresent = FALSE;
+GtkWidget *mouse_tray_icon = NULL;
+GtkWidget *mouse_tray_image = NULL;
+#endif
 
 static void get_mouse_values(int *accel_return, int *denom_return, int *thresh_return)
 {
@@ -74,13 +113,13 @@ static void get_mouse_values(int *accel_return, int *denom_return, int *thresh_r
     XChangePointerControl(GDK_DISPLAY(), True, True, -1, -1, -1);
     gdk_flush();
     XGetPointerControl(GDK_DISPLAY(), accel_return, denom_return, thresh_return);
-    
+
     /* Paranoid check */
     if (*denom_return < 1)
     {
         *denom_return = 1;
     }
-    
+
     gdk_flush();
     gdk_error_trap_pop();
 }
@@ -99,7 +138,7 @@ static void set_mouse_values(gboolean right_handed, int accel, int threshold)
     buttons = g_alloca (DEFAULT_PTR_MAP_SIZE);
     n_buttons = XGetPointerMapping (GDK_DISPLAY (), buttons, DEFAULT_PTR_MAP_SIZE);
 
-    if (n_buttons > DEFAULT_PTR_MAP_SIZE) 
+    if (n_buttons > DEFAULT_PTR_MAP_SIZE)
     {
         buttons = g_alloca (n_buttons);
         n_buttons = XGetPointerMapping (GDK_DISPLAY (), buttons, n_buttons);
@@ -206,14 +245,98 @@ static void cb_dbl_clicktime_changed(GtkWidget * dialog, gpointer user_data)
     mouse_plugin_write_options(mcs_plugin);
 }
 
+#ifdef USE_XKB
+void
+create_accessx_page(Itf *dialog)
+{
+    GtkWidget *label;
+    GtkWidget *vbox;
+    GtkWidget *hbox;
+    GtkWidget *main_vbox = gtk_vbox_new(FALSE, 5);
+
+    vbox = gtk_vbox_new (FALSE, 5);
+    gtk_widget_show (vbox);
+    gtk_box_pack_start (GTK_BOX (main_vbox), vbox, FALSE, FALSE, 0);
+
+    dialog->checkbutton_mouse = gtk_check_button_new_with_mnemonic(_("Enable mouse emulation"));
+    gtk_widget_show(dialog->checkbutton_mouse);
+    gtk_box_pack_start(GTK_BOX(vbox), dialog->checkbutton_mouse, FALSE, FALSE, 0);
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->checkbutton_mouse), mouse_key);
+
+    hbox = gtk_hbox_new(TRUE, 5);
+    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+    gtk_widget_show(hbox);
+
+    vbox = gtk_vbox_new(FALSE, 5);
+    gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);
+    gtk_widget_show(vbox);
+
+    label = gtk_label_new (_("Delay :"));
+    gtk_widget_show (label);
+    gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
+    gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
+    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
+
+    dialog->scale_mouse_keys_delay = gtk_hscale_new (GTK_ADJUSTMENT (gtk_adjustment_new (mouse_keys_delay, 10, 500, 10, 10, 0)));
+    gtk_widget_show(dialog->scale_mouse_keys_delay);
+    gtk_scale_set_draw_value (GTK_SCALE (dialog->scale_mouse_keys_delay), FALSE);
+    gtk_range_set_update_policy (GTK_RANGE (dialog->scale_mouse_keys_delay), GTK_UPDATE_DISCONTINUOUS);
+    gtk_box_pack_start(GTK_BOX(vbox), dialog->scale_mouse_keys_delay, FALSE, TRUE, 0);
+    gtk_widget_set_sensitive(dialog->scale_mouse_keys_delay, mouse_key);
+
+    label = gtk_label_new (_("Interval :"));
+    gtk_widget_show (label);
+    gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
+    gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
+    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
+
+    dialog->scale_mouse_keys_interval = gtk_hscale_new (GTK_ADJUSTMENT (gtk_adjustment_new (mouse_keys_interval, 10, 500, 10, 10, 0)));
+    gtk_widget_show(dialog->scale_mouse_keys_interval);
+    gtk_scale_set_draw_value (GTK_SCALE (dialog->scale_mouse_keys_interval), FALSE);
+    gtk_range_set_update_policy (GTK_RANGE (dialog->scale_mouse_keys_interval), GTK_UPDATE_DISCONTINUOUS);
+    gtk_box_pack_start(GTK_BOX(vbox), dialog->scale_mouse_keys_interval, FALSE, TRUE, 0);
+    gtk_widget_set_sensitive(dialog->scale_mouse_keys_interval, mouse_key);
+
+    vbox = gtk_vbox_new(FALSE, 5);
+    gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);
+    gtk_widget_show(vbox);
+
+    label = gtk_label_new (_("Time to max :"));
+    gtk_widget_show (label);
+    gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
+    gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
+    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
+
+    dialog->scale_mouse_keys_ttm= gtk_hscale_new (GTK_ADJUSTMENT (gtk_adjustment_new (mouse_keys_ttm, 10, 500, 10, 10, 0)));
+    gtk_widget_show(dialog->scale_mouse_keys_ttm);
+    gtk_scale_set_draw_value (GTK_SCALE (dialog->scale_mouse_keys_ttm), FALSE);
+    gtk_range_set_update_policy (GTK_RANGE (dialog->scale_mouse_keys_ttm), GTK_UPDATE_DISCONTINUOUS);
+    gtk_box_pack_start(GTK_BOX(vbox), dialog->scale_mouse_keys_ttm, FALSE, TRUE, 0);
+    gtk_widget_set_sensitive(dialog->scale_mouse_keys_ttm, mouse_key);
+
+    label = gtk_label_new (_("Max speed:"));
+    gtk_widget_show (label);
+    gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
+    gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
+    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
+
+    dialog->scale_mouse_keys_max_speed = gtk_hscale_new (GTK_ADJUSTMENT (gtk_adjustment_new (mouse_keys_max_speed, 10, 500, 10, 10, 0)));
+    gtk_widget_show(dialog->scale_mouse_keys_max_speed);
+    gtk_scale_set_draw_value (GTK_SCALE (dialog->scale_mouse_keys_max_speed), FALSE);
+    gtk_range_set_update_policy (GTK_RANGE (dialog->scale_mouse_keys_max_speed), GTK_UPDATE_DISCONTINUOUS);
+    gtk_box_pack_start(GTK_BOX(vbox), dialog->scale_mouse_keys_max_speed, FALSE, TRUE, 0);
+    gtk_widget_set_sensitive(dialog->scale_mouse_keys_max_speed, mouse_key);
+
+    dialog->accessx_page = main_vbox;
+}
+#endif
+
 Itf *create_mouse_dialog(McsPlugin * mcs_plugin)
 {
     Itf *dialog;
     GtkSizeGroup *sg;
     GtkWidget *frame_bin;
-#ifdef HAVE_XCURSOR_EXTENSION
-    GtkWidget *lbl;
-#endif
+    GtkWidget *label;
 
     dialog = g_new(Itf, 1);
 
@@ -235,23 +358,19 @@ Itf *create_mouse_dialog(McsPlugin * mcs_plugin)
     gtk_widget_show(dialog->notebook_vbox);
     gtk_box_pack_start(GTK_BOX(dialog->dialog_vbox1), dialog->notebook_vbox, TRUE, TRUE, 0);
 
-#ifdef HAVE_XCURSOR_EXTENSION
     dialog->notebook = gtk_notebook_new();
     gtk_widget_show(dialog->notebook);
     gtk_box_pack_start(GTK_BOX(dialog->notebook_vbox), dialog->notebook, TRUE, TRUE, 0);
-    
-    lbl = gtk_label_new(_("Behavior"));
-    gtk_widget_show(lbl);
+
+    label = gtk_label_new(_("Behavior"));
+    gtk_widget_show(label);
     dialog->vbox1 = gtk_vbox_new(FALSE, 6);
     gtk_container_set_border_width(GTK_CONTAINER(dialog->vbox1), 6);
     gtk_widget_show(dialog->vbox1);
-    gtk_notebook_append_page(GTK_NOTEBOOK(dialog->notebook), dialog->vbox1, lbl);
-#else
-    dialog->vbox1 = dialog->notebook_vbox;
-#endif
+    gtk_notebook_append_page(GTK_NOTEBOOK(dialog->notebook), dialog->vbox1, label);
 
     sg = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
-    
+
     /* button order */
     dialog->frame1 = xfce_create_framebox (_("Button settings"), &frame_bin);
     gtk_widget_show(dialog->frame1);
@@ -260,7 +379,7 @@ Itf *create_mouse_dialog(McsPlugin * mcs_plugin)
     dialog->hbox2 = gtk_hbox_new(FALSE, 0);
     gtk_widget_show(dialog->hbox2);
     gtk_container_add (GTK_CONTAINER (frame_bin), dialog->hbox2);
-    
+
     dialog->radiobutton1 = gtk_radio_button_new_with_mnemonic(NULL, _("Left Handed"));
     gtk_widget_show(dialog->radiobutton1);
     gtk_box_pack_start(GTK_BOX(dialog->hbox2), dialog->radiobutton1, FALSE, FALSE, 0);
@@ -418,15 +537,134 @@ Itf *create_mouse_dialog(McsPlugin * mcs_plugin)
     GTK_WIDGET_SET_FLAGS(dialog->closebutton1, GTK_CAN_DEFAULT);
 
 #ifdef HAVE_XCURSOR_EXTENSION
-    lbl = gtk_label_new(_("Cursor"));
-    gtk_widget_show(lbl);
+    label = gtk_label_new(_("Cursor"));
+    gtk_widget_show(label);
     mouse_plugin_create_cursor_page(dialog);
-    gtk_notebook_append_page(GTK_NOTEBOOK(dialog->notebook), dialog->cursor_page, lbl);
+    gtk_notebook_append_page(GTK_NOTEBOOK(dialog->notebook), dialog->cursor_page, label);
 #endif
-    
+
+#ifdef USE_XKB
+    label = gtk_label_new(_("Accessibility"));
+    gtk_widget_show(label);
+    create_accessx_page(dialog);
+    gtk_widget_show_all(dialog->accessx_page);
+    gtk_notebook_append_page(GTK_NOTEBOOK(dialog->notebook), dialog->accessx_page, label);
+#endif
+
     return dialog;
 }
 
+#ifdef USE_XKB
+static void
+cb_mouse_tray_icon_embedded(GtkWidget *tray_icon, gpointer user_data)
+{
+    gtk_container_add(GTK_CONTAINER(tray_icon), mouse_tray_image);
+    gtk_widget_show_all(tray_icon);
+}
+
+static void
+mouse_tray_icon_reconnect()
+{
+    mouse_tray_icon = netk_tray_icon_new(DefaultScreenOfDisplay(GDK_DISPLAY()));
+    g_signal_connect(G_OBJECT(mouse_tray_icon), "embedded", G_CALLBACK(cb_mouse_tray_icon_embedded), NULL);
+}
+
+static void
+cb_mouse_tray_icon_destroy(GtkWidget *tray_icon, GtkWidget *image)
+{
+    gtk_widget_ref(image);
+    gtk_container_remove(GTK_CONTAINER(tray_icon), image);
+    mouse_tray_icon_reconnect();
+}
+
+static void
+toggle_accessx()
+{
+    if (xkbpresent)
+    {
+        XkbDescPtr xkb = XkbAllocKeyboard ();
+        if (xkb)
+        {
+            gdk_error_trap_push ();
+            XkbGetControls (GDK_DISPLAY (), XkbAllControlsMask, xkb);
+
+            if(mouse_key)
+            {
+                xkb->ctrls->enabled_ctrls |= XkbMouseKeysMask;
+                xkb->ctrls->mk_delay = mouse_keys_delay;
+                xkb->ctrls->mk_interval = 1000 / mouse_keys_interval;
+                xkb->ctrls->mk_time_to_max = mouse_keys_ttm;
+                xkb->ctrls->mk_max_speed = mouse_keys_max_speed;
+
+                if(!mouse_tray_icon)
+                {
+                    GdkPixbuf *pixbuf = gdk_pixbuf_new_from_xpm_data(mouse_tray_image_xpm);
+                    mouse_tray_image = gtk_image_new_from_pixbuf(pixbuf);
+                    mouse_tray_icon = netk_tray_icon_new(DefaultScreenOfDisplay(GDK_DISPLAY()));
+                    gtk_container_add(GTK_CONTAINER(mouse_tray_icon), mouse_tray_image);
+                    g_signal_connect(G_OBJECT(mouse_tray_icon), "destroy", G_CALLBACK(cb_mouse_tray_icon_destroy), mouse_tray_image);
+                }
+                if(mouse_tray_icon)
+                    gtk_widget_show_all(mouse_tray_icon);
+            }
+            else
+            {
+                xkb->ctrls->enabled_ctrls &= ~XkbMouseKeysMask;
+                if(mouse_tray_icon)
+                    gtk_widget_hide(mouse_tray_icon);
+            }
+
+            XkbSetControls (GDK_DISPLAY (), XkbControlsEnabledMask | XkbMouseKeysMask | XkbMouseKeysAccelMask, xkb);
+            XFree (xkb);
+            gdk_flush ();
+            gdk_error_trap_pop ();
+        }
+        else
+        {
+            g_warning ("XkbAllocKeyboard() returned null pointer");
+        }
+    }
+}
+
+static void
+cb_scale_accessx_changed (GtkWidget * widget, gpointer user_data)
+{
+    Itf *itf = (Itf *)user_data;
+                McsPlugin *mcs_plugin = itf->mcs_plugin;
+
+    mouse_keys_delay = (int) gtk_range_get_value (GTK_RANGE (itf->scale_mouse_keys_delay));
+    mouse_keys_interval = (int) gtk_range_get_value (GTK_RANGE (itf->scale_mouse_keys_interval));
+    mouse_keys_ttm= (int) gtk_range_get_value (GTK_RANGE (itf->scale_mouse_keys_ttm));
+    mouse_keys_max_speed= (int) gtk_range_get_value (GTK_RANGE (itf->scale_mouse_keys_max_speed));
+
+    toggle_accessx();
+    mcs_manager_set_int (mcs_plugin->manager, "Mouse/MouseKeysDelay", CHANNEL2, mouse_keys_delay);
+    mcs_manager_set_int (mcs_plugin->manager, "Mouse/MouseKeysInterval", CHANNEL2, mouse_keys_interval);
+    mcs_manager_set_int (mcs_plugin->manager, "Mouse/MouseKeysTimeToMax", CHANNEL2, mouse_keys_ttm);
+    mcs_manager_set_int (mcs_plugin->manager, "Mouse/MouseKeysMaxSpeed", CHANNEL2, mouse_keys_max_speed);
+    mcs_manager_notify(mcs_plugin->manager, CHANNEL2);
+    mouse_plugin_write_options (mcs_plugin);
+}
+
+static void
+cb_checkbutton_accessx_changed (GtkWidget * widget, gpointer user_data)
+{
+    Itf *itf = (Itf *)user_data;
+    McsPlugin *mcs_plugin = itf->mcs_plugin;
+
+    mouse_key = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (itf->checkbutton_mouse));
+    gtk_widget_set_sensitive(itf->scale_mouse_keys_delay, mouse_key);
+    gtk_widget_set_sensitive(itf->scale_mouse_keys_interval, mouse_key);
+    gtk_widget_set_sensitive(itf->scale_mouse_keys_ttm, mouse_key);
+    gtk_widget_set_sensitive(itf->scale_mouse_keys_max_speed, mouse_key);
+
+    toggle_accessx();
+    mcs_manager_set_int (mcs_plugin->manager, "Mouse/MouseKeys", CHANNEL2, mouse_key ? 1 : 0);
+    mcs_manager_notify(mcs_plugin->manager, CHANNEL2);
+    mouse_plugin_write_options (mcs_plugin);
+}
+#endif
+
 static void setup_dialog(Itf * itf)
 {
     g_signal_connect(G_OBJECT(itf->mouse_dialog), "response", G_CALLBACK(cb_dialog_response), itf->mcs_plugin);
@@ -438,9 +676,18 @@ static void setup_dialog(Itf * itf)
     g_signal_connect(G_OBJECT(itf->hscale3), "value_changed", (GCallback) cb_dnd_threshold_changed, itf);
     g_signal_connect(G_OBJECT(itf->hscale4), "value_changed", (GCallback) cb_dbl_clicktime_changed, itf);
 
+#ifdef USE_XKB
+    g_signal_connect(G_OBJECT(itf->checkbutton_mouse), "toggled", (GCallback) cb_checkbutton_accessx_changed , itf);
+    g_signal_connect(G_OBJECT(itf->scale_mouse_keys_delay), "value_changed", (GCallback) cb_scale_accessx_changed , itf);
+    g_signal_connect(G_OBJECT(itf->scale_mouse_keys_interval), "value_changed", (GCallback) cb_scale_accessx_changed, itf);
+    g_signal_connect(G_OBJECT(itf->scale_mouse_keys_ttm), "value_changed", (GCallback) cb_scale_accessx_changed, itf);
+    g_signal_connect(G_OBJECT(itf->scale_mouse_keys_max_speed), "value_changed", (GCallback) cb_scale_accessx_changed , itf);
+#endif
+
+
     xfce_gtk_window_center_on_monitor_with_pointer (GTK_WINDOW (itf->mouse_dialog));
-    gdk_x11_window_set_user_time(GTK_WIDGET (itf->mouse_dialog)->window, 
-            gdk_x11_get_server_time (GTK_WIDGET (itf->mouse_dialog)->window));
+    gdk_x11_window_set_user_time(GTK_WIDGET (itf->mouse_dialog)->window,
+    gdk_x11_get_server_time (GTK_WIDGET (itf->mouse_dialog)->window));
     gtk_widget_show(itf->mouse_dialog);
 }
 
@@ -458,7 +705,7 @@ mcs_plugin_init(McsPlugin *mcs_plugin)
     mcs_manager_notify(mcs_plugin->manager, CHANNEL1);
     mcs_plugin->icon = xfce_themed_icon_load ("xfce4-mouse", 48);
     if (G_LIKELY (mcs_plugin->icon != NULL))
-      g_object_set_data_full (G_OBJECT (mcs_plugin->icon), "mcs-plugin-icon-name", g_strdup ("xfce4-mouse"), g_free);
+        g_object_set_data_full (G_OBJECT (mcs_plugin->icon), "mcs-plugin-icon-name", g_strdup ("xfce4-mouse"), g_free);
 
     return(MCS_PLUGIN_INIT_OK);
 }
@@ -468,7 +715,11 @@ create_channel(McsPlugin *mcs_plugin)
 {
     McsSetting *setting;
     gchar *rcfile, *path;
-    
+#ifdef USE_XKB
+    int xkbmajor = XkbMajorVersion, xkbminor = XkbMinorVersion;
+    int xkbopcode, xkbevent, xkberror;
+#endif
+
     path = g_build_filename ("xfce4", RCDIR, RCFILE1, NULL);
     rcfile = xfce_resource_lookup (XFCE_RESOURCE_CONFIG, path);
 
@@ -479,14 +730,14 @@ create_channel(McsPlugin *mcs_plugin)
 
     if (g_file_test (rcfile, G_FILE_TEST_EXISTS))
     {
-        mcs_manager_add_channel_from_file(mcs_plugin->manager, CHANNEL1, 
+        mcs_manager_add_channel_from_file(mcs_plugin->manager, CHANNEL1,
                                           rcfile);
     }
     else
     {
         mcs_manager_add_channel(mcs_plugin->manager, CHANNEL1);
     }
-    
+
     g_free(path);
     g_free(rcfile);
 
@@ -500,24 +751,24 @@ create_channel(McsPlugin *mcs_plugin)
 
     if (g_file_test (rcfile, G_FILE_TEST_EXISTS))
     {
-        mcs_manager_add_channel_from_file(mcs_plugin->manager, CHANNEL2, 
-                                          rcfile);
+        mcs_manager_add_channel_from_file(mcs_plugin->manager, CHANNEL2, rcfile);
     }
     else
     {
         mcs_manager_add_channel(mcs_plugin->manager, CHANNEL2);
     }
-    
+
     g_free(path);
     g_free(rcfile);
 
     /* Retrieve server defaults */
     get_mouse_values(&acceleration, &denominator, &threshold);
-    
-    setting = mcs_manager_setting_lookup(mcs_plugin->manager,
-                    "Net/DoubleClickTime", CHANNEL1);
+
+    setting = mcs_manager_setting_lookup(mcs_plugin->manager, "Net/DoubleClickTime", CHANNEL1);
     if (setting)
+    {
         dbl_clicktime = setting->data.v_int;
+    }
     else
     {
         dbl_clicktime = DEFAULT_DBL_CLICKTIME;
@@ -578,13 +829,88 @@ create_channel(McsPlugin *mcs_plugin)
         mcs_manager_set_int(mcs_plugin->manager, "Mouse/Threshold", CHANNEL2, threshold);
     }
 
+#ifdef USE_XKB
+#ifdef DEBUG
+    g_message ("Querying Xkb extension");
+#endif
+    if (XkbQueryExtension (GDK_DISPLAY (), &xkbopcode, &xkbevent, &xkberror, &xkbmajor, &xkbminor))
+    {
+#ifdef DEBUG
+        g_message ("Xkb extension found");
+#endif
+        xkbpresent = TRUE;
+    }
+    else
+    {
+#ifdef DEBUG
+        g_message ("Your X server does not support Xkb extension");
+#endif
+        xkbpresent = FALSE;
+    }
+    setting = mcs_manager_setting_lookup (mcs_plugin->manager, "Mouse/MouseKeys", CHANNEL2);
+    if(setting)
+    {
+        mouse_key = (setting->data.v_int ? TRUE : FALSE);
+    }
+    else
+    {
+        mouse_key = FALSE;
+        mcs_manager_set_int (mcs_plugin->manager, "Mouse/MouseKeys", CHANNEL2, mouse_key ? 1 : 0);
+    }
+
+    setting = mcs_manager_setting_lookup (mcs_plugin->manager, "Mouse/MouseKeysDelay", CHANNEL2);
+    if(setting)
+    {
+        mouse_keys_delay = setting->data.v_int;
+    }
+    else
+    {
+        mouse_keys_delay = 200;
+        mcs_manager_set_int (mcs_plugin->manager, "Mouse/MouseKeysDelay", CHANNEL2, mouse_keys_delay);
+    }
+
+    setting = mcs_manager_setting_lookup (mcs_plugin->manager, "Mouse/MouseKeysInterval", CHANNEL2);
+    if(setting)
+    {
+        mouse_keys_interval = setting->data.v_int;
+    }
+    else
+    {
+        mouse_keys_interval = 200;
+        mcs_manager_set_int (mcs_plugin->manager, "Mouse/MouseKeysInterval", CHANNEL2, mouse_keys_interval);
+    }
+
+    setting = mcs_manager_setting_lookup (mcs_plugin->manager, "Mouse/MouseKeysMaxSpeed", CHANNEL2);
+    if(setting)
+    {
+        mouse_keys_max_speed = setting->data.v_int;
+    }
+    else
+    {
+        mouse_keys_max_speed = 200;
+        mcs_manager_set_int (mcs_plugin->manager, "Mouse/MouseKeysMaxSpeed", CHANNEL2, mouse_keys_max_speed);
+    }
+
+    setting = mcs_manager_setting_lookup (mcs_plugin->manager, "Mouse/MouseKeysTimeToMax", CHANNEL2);
+    if(setting)
+    {
+        mouse_keys_ttm = setting->data.v_int;
+    }
+    else
+    {
+        mouse_keys_ttm = 200;
+        mcs_manager_set_int (mcs_plugin->manager, "Mouse/MouseKeysTimeToMax", CHANNEL2, mouse_keys_ttm);
+    }
+
+    toggle_accessx();
+#endif
     set_mouse_values(right_handed, acceleration, threshold);
-    
+
 #ifdef HAVE_XCURSOR_EXTENSION
     mouse_plugin_set_initial_cursor_values(mcs_plugin);
-#endif    
+#endif
 
-#if 0    
+#if 0
     /* I fail to see why we need to save the options here, during startup... */
     mouse_plugin_write_options (mcs_plugin);
 #endif
@@ -596,7 +922,7 @@ mouse_plugin_write_options(McsPlugin *mcs_plugin)
     gboolean result = FALSE;
     gchar *rcfile, *path;
 
-    path = g_build_filename ("xfce4", RCDIR, RCFILE1, NULL);    
+    path = g_build_filename ("xfce4", RCDIR, RCFILE1, NULL);
     rcfile = xfce_resource_save_location (XFCE_RESOURCE_CONFIG, path, TRUE);
     if (G_LIKELY (rcfile != NULL))
     {
@@ -605,7 +931,7 @@ mouse_plugin_write_options(McsPlugin *mcs_plugin)
     }
     g_free (path);
 
-    path = g_build_filename ("xfce4", RCDIR, RCFILE2, NULL);    
+    path = g_build_filename ("xfce4", RCDIR, RCFILE2, NULL);
     rcfile = xfce_resource_save_location (XFCE_RESOURCE_CONFIG, path, TRUE);
     if (G_LIKELY (rcfile != NULL))
     {
